/* IOHandler.h - TransparentIRC 0.1
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#ifndef _IOHandler_h
#define _IOHandler_h
#include "overall.h"

struct IODescriptor;
struct IOEvent;

#define IOHANDLER_CALLBACK(NAME) void NAME(UNUSED_ARG(struct IOEvent *io_event))
typedef IOHANDLER_CALLBACK(iohandler_callback);

enum IOType {
    IOTYPE_UNKNOWN, /* ignore descriptor (uninitialized) */
    IOTYPE_SERVER, /* server socket */
    IOTYPE_CLIENT, /* client socket */
    IOTYPE_STDIN, /* stdin */
    IOTYPE_TIMER /* timer */
};

enum IOStatus { 
    IO_CLOSED, /* descriptor is dead (socket waiting for removal or timer) */
    IO_LISTENING, /* descriptor is waiting for connections (server socket) */
    IO_CONNECTING, /* descriptor is waiting for connection approval (connecting client socket) */
    IO_CONNECTED /* descriptor is connected (connected client socket) */
};

enum IOEventType {
    IOEVENT_IGNORE,
    IOEVENT_READABLE, /* socket is readable - not read anything yet, could also be disconnect notification */
    IOEVENT_RECV, /* client socket received something (recv_str valid) */
    IOEVENT_CONNECTED, /* client socket connected successful */
    IOEVENT_NOTCONNECTED, /* client socket could not connect (errno valid) */
    IOEVENT_CLOSED, /* client socket lost connection (errno valid) */
    IOEVENT_ACCEPT, /* server socket accepted new connection (accept_fd valid) */
    IOEVENT_TIMEOUT /* timer timed out */
};

struct IOBuffer {
    char buffer;
    size_t bufpos, buflen;
};

struct IODescriptor {
    int fd;
    IOType type;
    IOStatus state;
    struct timeval timeout;
    iohandler_callback *callback;
    struct IOBuffer readbuf;
    struct IOBuffer writebuf;
    void *data;
    int read_lines : 1;
    
    struct IODescriptor *next, *prev;
};

struct IOEvent {
    IOEventType type;
    struct IODescriptor *iofd;
    union {
        char *recv_str;
        int accept_fd;
        int errno;
    } data;
};

struct IODescriptor *iohandler_add(int sockfd, IOType type, iohandler_callback *callback);
struct IODescriptor *iohandler_timer(timeval timeout, iohandler_callback *callback);
struct IODescriptor *iohandler_connect(const char *hostname, unsigned int port, const char *bind, iohandler_callback *callback);
struct IODescriptor *iohandler_listen(const char *hostname, unsigned int port, iohandler_callback *callback);
void iohandler_write(struct IODescriptor *iofd, const char *line);
void iohandler_send(struct IODescriptor *iofd, const char *data, size_t datalen);
void iohandler_printf(struct IODescriptor *iofd, const char *text, ...);
void iohandler_close(struct IODescriptor *iofd);
void iohandler_update(struct IODescriptor *iofd);

void iohandler_poll();

#endif
