/* ServerSocket.c - TransparentIRC 0.1
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#include "ServerSocket.h"
#include "IOHandler.h"
#include "UserClient.h"

static struct ServerSocket *serversockets = NULL;

static void serversocket_callback(struct IOEvent *event);

static struct ServerSocket *serversocket_create(struct IODescriptor *iofd) {
    struct ServerSocket *sock = malloc(sizeof(*sock));
    sock->iofd = iofd;
    sock->clientcount = 0;
    
    //add ServerSocket to the list
    sock->prev = NULL;
    sock->next = serversockets;
    serversockets->prev = sock;
    serversockets = sock;
}

static void serversocket_delete(struct ServerSocket *server) {
    if(server->prev)
        server->prev->next = server->next;
    else
        serversockets = server->next;
    if(server->next)
        server->next->prev = server->prev;
    free(server);
}

struct ServerSocket *serversocket_listen(char *hostname, int port) {
    struct IODescriptor *iofd = iohandler_listen(hostname, port, serversocket_callback);
    if(iofd) {
        struct ServerSocket *server = serversocket_create(iofd);
        return server;
    }
    return NULL;
}

void serversocket_close(struct ServerSocket *server, int keep_clients) {
    userclient_close_server(server, keep_clients);
    iohandler_close(server->iofd);
    serversocket_delete(server);
}

static void serversocket_callback(struct IOEvent *event) {
    switch(event->type) {
        case IOEVENT_ACCEPT:
            struct ServerSocket *server = NULL;
            for(server = serversockets; server; server = server->next) {
                if(server->iofd == event->iofd)
                    break;
            }
            if(server)
                userclient_accepted(server, event->accept_fd)
            break;
    }
}

