/* UserClient.c - TransparentIRC 0.1
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */
#include "UserClient.c"
#include "IOHandler.h"

static void userclient_callback(struct IOEvent *event);

static struct UserClient *userclients = NULL;

void userclient_accepted(struct ServerSocket *server, int sockfd) {
    struct UserClient *client;
    struct IODescriptor *iofd = iohandler_add(sockfd, IOTYPE_CLIENT, userclient_callback);
    if(!iofd) return;
    iofd->state = IO_CONNECTED;
    iofd->read_lines = 1;
    iohandler_update(iofd);
    client = malloc(sizeof(*client));
    client->iofd = iofd;
    client->server = server;
    server->clientcount++;
    
    //add UserClient to the list
    sock->prev = NULL;
    sock->next = serversockets;
    serversockets->prev = sock;
    serversockets = sock;
    
    //let's say hello to the client
    iohandler_printf(iofd, "NOTICE AUTH :*** TransparentIRC " TRANSIRC_VERSION " (use /quote transirc for more information)");
}

void userclient_close(struct UserClient *client) {
    iohandler_close(client->iofd);
    client->server->clientcount--;
    if(client->prev)
        client->prev->next = client->next;
    else
        userclients = client->next;
    if(client->next)
        client->next->prev = client->prev;
    free(client);
}

void userclient_close_server(struct ServerSocket *server, int keep_clients) {
    struct UserClient *client, *next_client;
    for(client = userclients; client; client = next_client) {
        if(client->server == server) {
            if(keep_clients)
                client->server = NULL;
            else
                userclient_close(client);
        }
    }
}

static void userclient_callback(struct IOEvent *event) {
    switch(event->type) {
        case IOEVENT_RECV:
            char *line = event->data.recv_str;
            iohandler_printf(event->iofd, "reply: %s", line);
            break;
        case IOEVENT_CLOSED:
            userclient_close(event->iofd);
            break;
    }
}
